#include <stdio.h>
#include "csapi.h"
#include "common.h"

int check_return_code( struct CSAPIState *s, int return_code, char *operation );

int main()
{
    DRVErrno return_code;
    struct CSAPIState *s;

    // First call CSAPI_new to load the CSAPI library.
    // This will return us a pointer that can be used with one board.
    s = CSAPI_new( CM_Direct );

    if( s == NULL )
    {
        printf("Error: Failed to create a state object.");
        return 10;
    }


    // Try to connect to any available board.
    // The return code must be checked to ensure we are connected.
    return_code = CSAPI_connect( s , HOST_NAME , CSAPI_INSTANCE_ANY );

    if( check_return_code( s, return_code, "connect" ) )
        return 20;


    // Load the csx application on to the board.
    return_code = CSAPI_load( s , 0 , CSX_FILE_NAME );

    if( check_return_code( s, return_code, "load" ) )
        return 30;


    // Run the loaded application
    return_code = CSAPI_run( s , 0 );

    if( check_return_code( s, return_code, "run" ) )
        return 40;


    // Wait for the application to terminate
    return_code = CSAPI_wait_on_terminate( s , 0 );

    if( check_return_code( s, return_code, "wait on terminate" ) )
        return 50;


    // Destroy the state to disconnect from the board an unload the CSAPI library
    CSAPI_delete( s );

    return 0;
}

// This function checks the return code from a CSAPI function. If there is an
// error it is decoded and printed along with the operation string. The function
// then destroys the CSAPI state object and returns 1 so that the program can exit
int check_return_code( struct CSAPIState *s, int return_code, char *operation )
{
    char error_string[100];
    if( return_code != DRVErrno_success )
    {
        CSAPI_get_error_string( s, return_code, error_string, 100 );
        printf("Error: Failed to %s: %s\n", operation, error_string );
        CSAPI_delete( s );
        return 1;
    }
    return 0;
}
